==> create-fly-1.21.8-6.0.8-2-sources.jar: com/zurrtum/create/content/trains/track/TrackBlockItem.java <==
package com.zurrtum.create.content.trains.track;

import com.zurrtum.create.AllDataComponents;
import com.zurrtum.create.AllItemTags;
import com.zurrtum.create.AllItems;
import com.zurrtum.create.AllSoundEvents;
import com.zurrtum.create.catnip.data.Pair;
import com.zurrtum.create.content.trains.track.TrackPlacement.PlacementInfo;
import com.zurrtum.create.infrastructure.component.ConnectingFrom;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1838;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350.class_2352;
import net.minecraft.class_243;
import net.minecraft.class_2498;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_3417;
import net.minecraft.class_3419;

public class TrackBlockItem extends class_1747 {

    public TrackBlockItem(class_2248 pBlock, class_1793 pProperties) {
        super(pBlock, pProperties);
    }

    @Override
    public class_1269 method_7836(class_1937 world, class_1657 player, class_1268 usedHand) {
        class_1799 stack = player.method_5998(usedHand);
        if (player.method_5715() && method_7886(stack)) {
            return clearSelection(stack, world, player);
        } else {
            return super.method_7836(world, player, usedHand);
        }
    }

    @Override
    public class_1269 method_7884(class_1838 pContext) {
        class_1799 stack = pContext.method_8041();
        class_2338 pos = pContext.method_8037();
        class_1937 level = pContext.method_8045();
        class_2680 state = level.method_8320(pos);
        class_1657 player = pContext.method_8036();

        if (player == null)
            return super.method_7884(pContext);
        if (pContext.method_20287() == class_1268.field_5810)
            return super.method_7884(pContext);

        class_243 lookAngle = player.method_5720();

        if (!method_7886(stack)) {
            if (state.method_26204() instanceof TrackBlock track && track.getTrackAxes(level, pos, state).size() > 1) {
                if (!level.field_9236)
                    player.method_7353(class_2561.method_43471("create.track.junction_start").method_27692(class_124.field_1061), true);
                return class_1269.field_5812;
            }

            if (level.method_8321(pos) instanceof TrackBlockEntity tbe && tbe.isTilted()) {
                if (!level.field_9236)
                    player.method_7353(class_2561.method_43471("create.track.turn_start").method_27692(class_124.field_1061), true);
                return class_1269.field_5812;
            }

            if (select(level, pos, lookAngle, stack)) {
                level.method_8396(null, pos, class_3417.field_14667, class_3419.field_15245, 0.75f, 1);
                return class_1269.field_5812;
            }
            return super.method_7884(pContext);

        } else if (player.method_5715()) {
            return clearSelection(stack, level, player);
        }

        boolean placing = !(state.method_26204() instanceof ITrackBlock);
        boolean extend = stack.method_58695(AllDataComponents.TRACK_EXTENDED_CURVE, false);
        stack.method_57381(AllDataComponents.TRACK_EXTENDED_CURVE);

        if (placing) {
            if (!state.method_45474())
                pos = pos.method_10093(pContext.method_8038());
            state = getPlacementState(pContext);
            if (state == null)
                return class_1269.field_5814;
        }

        class_1799 offhandItem = player.method_6079();
        boolean hasGirder = offhandItem.method_31574(AllItems.METAL_GIRDER);
        PlacementInfo info = TrackPlacement.tryConnect(level, player, pos, state, stack, hasGirder, extend);

        if (info.message != null && !level.field_9236)
            player.method_7353(class_2561.method_43471("create." + info.message), true);
        if (!info.valid) {
            AllSoundEvents.DENY.playFrom(player, 1, 1);
            return class_1269.field_5814;
        }

        if (level.field_9236)
            return class_1269.field_5812;

        stack = player.method_6047();
        if (stack.method_31573(AllItemTags.TRACKS)) {
            stack.method_57381(AllDataComponents.TRACK_CONNECTING_FROM);
            stack.method_57381(AllDataComponents.TRACK_EXTENDED_CURVE);
            player.method_6122(pContext.method_20287(), stack);
        }

        class_2498 soundtype = state.method_26231();
        if (soundtype != null)
            level.method_8396(
                null,
                pos,
                soundtype.method_10598(),
                class_3419.field_15245,
                (soundtype.method_10597() + 1.0F) / 2.0F,
                soundtype.method_10599() * 0.8F
            );

        return class_1269.field_5812;
    }

    public static class_1269 clearSelection(class_1799 stack, class_1937 level, class_1657 player) {
        if (level.field_9236) {
            level.method_8396(player, player.method_24515(), class_3417.field_14770, class_3419.field_15245, 0.75f, 1.0f);
        } else {
            player.method_7353(class_2561.method_43471("create.track.selection_cleared"), true);
            stack.method_57381(AllDataComponents.TRACK_CONNECTING_FROM);
        }
        return class_1269.field_5812.method_61393(stack);
    }

    public class_2680 getPlacementState(class_1838 pContext) {
        return method_7707(method_16356(new class_1750(pContext)));
    }

    public static boolean select(class_1936 world, class_2338 pos, class_243 lookVec, class_1799 heldItem) {
        class_2680 blockState = world.method_8320(pos);
        class_2248 block = blockState.method_26204();
        if (!(block instanceof ITrackBlock track))
            return false;

        Pair<class_243, class_2352> nearestTrackAxis = track.getNearestTrackAxis(world, pos, blockState, lookVec);
        class_243 axis = nearestTrackAxis.getFirst().method_1021(nearestTrackAxis.getSecond() == class_2352.field_11056 ? -1 : 1);
        class_243 end = track.getCurveStart(world, pos, blockState, axis);
        class_243 normal = track.getUpNormal(world, pos, blockState).method_1029();

        heldItem.method_57379(AllDataComponents.TRACK_CONNECTING_FROM, new ConnectingFrom(pos, axis, normal, end));
        return true;
    }

    @Override
    public boolean method_7886(class_1799 stack) {
        return stack.method_57826(AllDataComponents.TRACK_CONNECTING_FROM);
    }

}

==> create-fly-1.21.11-6.0.8-2-sources.jar: com/zurrtum/create/content/trains/track/TrackBlockItem.java <==
package com.zurrtum.create.content.trains.track;

import com.zurrtum.create.AllDataComponents;
import com.zurrtum.create.AllItemTags;
import com.zurrtum.create.AllItems;
import com.zurrtum.create.AllSoundEvents;
import com.zurrtum.create.catnip.data.Pair;
import com.zurrtum.create.content.trains.track.TrackPlacement.PlacementInfo;
import com.zurrtum.create.infrastructure.component.ConnectingFrom;
import net.minecraft.class_124;
import net.minecraft.class_1268;
import net.minecraft.class_1269;
import net.minecraft.class_1657;
import net.minecraft.class_1747;
import net.minecraft.class_1750;
import net.minecraft.class_1799;
import net.minecraft.class_1838;
import net.minecraft.class_1936;
import net.minecraft.class_1937;
import net.minecraft.class_2248;
import net.minecraft.class_2338;
import net.minecraft.class_2350.class_2352;
import net.minecraft.class_243;
import net.minecraft.class_2498;
import net.minecraft.class_2561;
import net.minecraft.class_2680;
import net.minecraft.class_3417;
import net.minecraft.class_3419;

public class TrackBlockItem extends class_1747 {

    public TrackBlockItem(class_2248 pBlock, class_1793 pProperties) {
        super(pBlock, pProperties);
    }

    @Override
    public class_1269 method_7836(class_1937 world, class_1657 player, class_1268 usedHand) {
        class_1799 stack = player.method_5998(usedHand);
        if (player.method_5715() && method_7886(stack)) {
            return clearSelection(stack, world, player);
        } else {
            return super.method_7836(world, player, usedHand);
        }
    }

    @Override
    public class_1269 method_7884(class_1838 pContext) {
        class_1799 stack = pContext.method_8041();
        class_2338 pos = pContext.method_8037();
        class_1937 level = pContext.method_8045();
        class_2680 state = level.method_8320(pos);
        class_1657 player = pContext.method_8036();

        if (player == null)
            return super.method_7884(pContext);
        if (pContext.method_20287() == class_1268.field_5810)
            return super.method_7884(pContext);

        class_243 lookAngle = player.method_5720();

        if (!method_7886(stack)) {
            if (state.method_26204() instanceof TrackBlock track && track.getTrackAxes(level, pos, state).size() > 1) {
                if (!level.method_8608())
                    player.method_7353(class_2561.method_43471("create.track.junction_start").method_27692(class_124.field_1061), true);
                return class_1269.field_5812;
            }

            if (level.method_8321(pos) instanceof TrackBlockEntity tbe && tbe.isTilted()) {
                if (!level.method_8608())
                    player.method_7353(class_2561.method_43471("create.track.turn_start").method_27692(class_124.field_1061), true);
                return class_1269.field_5812;
            }

            if (select(level, pos, lookAngle, stack)) {
                level.method_8396(null, pos, class_3417.field_14667, class_3419.field_15245, 0.75f, 1);
                return class_1269.field_5812;
            }
            return super.method_7884(pContext);

        } else if (player.method_5715()) {
            return clearSelection(stack, level, player);
        }

        boolean placing = !(state.method_26204() instanceof ITrackBlock);
        boolean extend = stack.method_58695(AllDataComponents.TRACK_EXTENDED_CURVE, false);
        stack.method_57381(AllDataComponents.TRACK_EXTENDED_CURVE);

        if (placing) {
            if (!state.method_45474())
                pos = pos.method_10093(pContext.method_8038());
            state = getPlacementState(pContext);
            if (state == null)
                return class_1269.field_5814;
        }

        class_1799 offhandItem = player.method_6079();
        boolean hasGirder = offhandItem.method_31574(AllItems.METAL_GIRDER);
        PlacementInfo info = TrackPlacement.tryConnect(level, player, pos, state, stack, hasGirder, extend);

        if (info.message != null && !level.method_8608())
            player.method_7353(class_2561.method_43471("create." + info.message), true);
        if (!info.valid) {
            AllSoundEvents.DENY.playFrom(player, 1, 1);
            return class_1269.field_5814;
        }

        if (level.method_8608())
            return class_1269.field_5812;

        stack = player.method_6047();
        if (stack.method_31573(AllItemTags.TRACKS)) {
            stack.method_57381(AllDataComponents.TRACK_CONNECTING_FROM);
            stack.method_57381(AllDataComponents.TRACK_EXTENDED_CURVE);
            player.method_6122(pContext.method_20287(), stack);
        }

        class_2498 soundtype = state.method_26231();
        if (soundtype != null)
            level.method_8396(
                null,
                pos,
                soundtype.method_10598(),
                class_3419.field_15245,
                (soundtype.method_10597() + 1.0F) / 2.0F,
                soundtype.method_10599() * 0.8F
            );

        return class_1269.field_5812;
    }

    public static class_1269 clearSelection(class_1799 stack, class_1937 level, class_1657 player) {
        if (level.method_8608()) {
            level.method_8396(player, player.method_24515(), class_3417.field_14770, class_3419.field_15245, 0.75f, 1.0f);
        } else {
            player.method_7353(class_2561.method_43471("create.track.selection_cleared"), true);
            stack.method_57381(AllDataComponents.TRACK_CONNECTING_FROM);
        }
        return class_1269.field_5812.method_61393(stack);
    }

    public class_2680 getPlacementState(class_1838 pContext) {
        return method_7707(method_16356(new class_1750(pContext)));
    }

    public static boolean select(class_1936 world, class_2338 pos, class_243 lookVec, class_1799 heldItem) {
        class_2680 blockState = world.method_8320(pos);
        class_2248 block = blockState.method_26204();
        if (!(block instanceof ITrackBlock track))
            return false;

        Pair<class_243, class_2352> nearestTrackAxis = track.getNearestTrackAxis(world, pos, blockState, lookVec);
        class_243 axis = nearestTrackAxis.getFirst().method_1021(nearestTrackAxis.getSecond() == class_2352.field_11056 ? -1 : 1);
        class_243 end = track.getCurveStart(world, pos, blockState, axis);
        class_243 normal = track.getUpNormal(world, pos, blockState).method_1029();

        heldItem.method_57379(AllDataComponents.TRACK_CONNECTING_FROM, new ConnectingFrom(pos, axis, normal, end));
        return true;
    }

    @Override
    public boolean method_7886(class_1799 stack) {
        return stack.method_57826(AllDataComponents.TRACK_CONNECTING_FROM);
    }

}
